@@ -3,6 +3,7 @@ from covid19_outbreak_simulator.plugin import BasePlugin
 from covid19_outbreak_simulator.utils import parse_param_with_multiplier
 import numpy
 
+
 class testing(BasePlugin):
 
     def __init__(self, *args, **kwargs):
@@ -14,7 +15,9 @@ class testing(BasePlugin):
         parser.prog = '--plugin testing'
         parser.description = '''PCR-based test that can pick out all active cases.'''
         parser.add_argument(
-            'IDs', nargs='*', help='''IDs of individuals to test. Parameter "proportion"
+            'IDs',
+            nargs='*',
+            help='''IDs of individuals to test. Parameter "proportion"
             will be ignored with specified IDs for testing''')
         parser.add_argument(
             '--proportion',
@@ -23,6 +26,11 @@ class testing(BasePlugin):
             positive will by default be quarantined. Multipliers are allowed to specify
             proportion of tests for each group.''',
         )
+        parser.add_argument(
+            '--ignore-vaccinated',
+            action='store_true',
+            help='''Do not test people who are vaccinated.''',
+        )
         parser.add_argument(
             '--sensitivity',
             nargs='+',
@@ -52,7 +60,8 @@ class testing(BasePlugin):
             '--handle-positive',
             default='remove',
             help='''How to handle individuals who are tested positive, which should be
-                "keep" (do not do anything), "remove" (remove from population), and "quarantine"
+                "keep" (do not do anything), "replace" (remove from population), "recover"
+                (instant recover, to model constant workforce size), and "quarantine"
                 (put aside until it recovers). Quarantine can be specified as "quarantine_7" etc
                 to specify  duration of quarantine. Individuals that are already in quarantine
                 will continue to be quarantined. Default to "remove", meaning all symptomatic cases
@@ -62,6 +71,8 @@ class testing(BasePlugin):
     def apply(self, time, population, args=None):
         n_infected = 0
         n_uninfected = 0
+        n_ignore_infected = 0
+        n_ignore_uninfected = 0
         n_recovered = 0
         n_false_positive = 0
         n_false_negative = 0
@@ -73,6 +84,8 @@ class testing(BasePlugin):
             nonlocal n_tested
             nonlocal n_infected
             nonlocal n_uninfected
+            nonlocal n_ignore_infected
+            nonlocal n_ignore_uninfected
             nonlocal n_recovered
             nonlocal n_false_positive
             nonlocal n_false_negative
@@ -84,10 +97,19 @@ class testing(BasePlugin):
                     counts[ind.group] -= 1
                 else:
                     return False
-            n_tested += 1
+
             affected = isinstance(ind.infected, float)
+            if ind.vaccinated is not False and args.ignore_vaccinated:
+                if affected:
+                    n_ignore_infected += 1
+                else:
+                    n_ignore_uninfected += 1
+                return False
+
+            n_tested += 1
             if affected:
-                test_lod = args.sensitivity[1] if len(args.sensitivity) == 2 else 0
+                test_lod = args.sensitivity[1] if len(
+                    args.sensitivity) == 2 else 0
                 lod_sensitivity = ind.test_sensitivity(time, test_lod)
                 #
                 sensitivity = lod_sensitivity * args.sensitivity[0]
@@ -107,7 +129,8 @@ class testing(BasePlugin):
             else:
                 n_uninfected += 1
 
-                res = args.specificity != 1 and args.specificity <= numpy.random.uniform()
+                res = args.specificity != 1 and args.specificity <= numpy.random.uniform(
+                )
                 if res:
                     n_false_positive += 1
                 return res
@@ -115,14 +138,16 @@ class testing(BasePlugin):
         if args.IDs:
             IDs = [x for x in args.IDs if select(population[x])]
         else:
-            proportions = parse_param_with_multiplier(args.proportion,
-                subpops=population.group_sizes.keys(), default=1.0)
-            counts = {name:int(size*proportions[name]) for name,size in population.group_sizes.items()}
+            proportions = parse_param_with_multiplier(
+                args.proportion,
+                subpops=population.group_sizes.keys(),
+                default=1.0)
+            counts = {
+                name: int(size * proportions[name])
+                for name, size in population.group_sizes.items()
+            }
 
-            IDs = [
-                x for x, y in population.items()
-                if select(y, counts)
-            ]
+            IDs = [x for x, y in population.items() if select(y, counts)]
 
         #print(f'SELECT {" ".join(IDs)}')
         events = []
@@ -133,7 +158,10 @@ class testing(BasePlugin):
             if args.handle_positive == 'remove':
                 events.append(
                     Event(
-                        time + args.turnaround_time, EventType.REMOVAL, target=ID, logger=self.logger))
+                        time + args.turnaround_time,
+                        EventType.REMOVAL,
+                        target=ID,
+                        logger=self.logger))
             elif args.handle_positive.startswith('quarantine'):
                 if args.handle_positive == 'quarantine':
                     duration = 14
@@ -147,6 +175,15 @@ class testing(BasePlugin):
                             target=ID,
                             logger=self.logger,
                             till=time + duration))
+            elif args.handle_positive == 'replace':
+                events.append(
+                    Event(
+                        time + args.turnaround_time,
+                        EventType.REPLACEMENT,
+                        reason='detected',
+                        keep=['vaccinated'],
+                        target=ID,
+                        logger=self.logger))
             elif args.handle_positive != 'keep':
                 raise ValueError(
                     'Unsupported action for patients who test positive.')
@@ -155,16 +192,17 @@ class testing(BasePlugin):
             n_tested=n_tested,
             n_infected=n_infected,
             n_uninfected=n_uninfected,
+            n_ignore_infected=n_ignore_infected,
+            n_ignore_uninfected=n_ignore_uninfected,
             n_recovered=n_recovered,
             n_detected=len(IDs),
             n_false_positive=n_false_positive,
             n_false_negative=n_false_negative,
             n_false_negative_lod=n_false_negative_lod,
-            n_false_negative_in_recovered=n_false_negative_in_recovered
-        )
+            n_false_negative_in_recovered=n_false_negative_in_recovered)
         if IDs and args.verbosity > 1:
             res['detected_IDs'] = ",".join(IDs)
-        res_str = ','.join(f'{k}={v}' for k,v in res.items())
+        res_str = ','.join(f'{k}={v}' for k, v in res.items())
         if args.verbosity > 0:
             self.logger.write(
                 f'{time:.2f}\t{EventType.PLUGIN.name}\t.\tname=testing,{res_str}\n'
